@@ -9,11 +9,13 @@ from django_generic_flatblocks.models import GenericFlatblock
 register = Library()
 
 class GenericFlatblockNode(Node):
-    def __init__(self, slug, modelname=None, template_path=None, variable_name=None):
+    def __init__(self, slug, modelname=None, template_path=None,
+                 variable_name=None, store_in_object=None):
         self.slug = slug
         self.modelname = modelname
         self.template_path = template_path
         self.variable_name = variable_name
+        self.store_in_object = store_in_object
 
     def generate_slug(self, slug, context):
         """
@@ -27,7 +29,7 @@ class GenericFlatblockNode(Node):
         # self.get_content_object()
         if not ',' in slug and isinstance(self.resolve(slug, context), int):
             return self.resolve(slug, context)
-        return slugify('_'.join([self.resolve(i, context) for i in slug.split(',')]))
+        return slugify('_'.join([str(self.resolve(i, context)) for i in slug.split(',')]))
 
     def generate_admin_link(self, related_object, context):
         """
@@ -49,13 +51,23 @@ class GenericFlatblockNode(Node):
         # If the user passed a Integer as a slug, assume that we should fetch
         # this specific object
         if isinstance(slug, int):
-            related_object = related_model._default_manager.get(pk=slug)
-            return None, related_object
+            try:
+                related_object = related_model._default_manager.get(pk=slug)
+                return None, related_object
+            except related_model.DoesNotExist:
+                if settings.TEMPLATE_DEBUG:
+                    raise
+                related_object = related_model()
+                return None, related_object
 
         # Otherwise, try to generate a new, related object
         try:
             generic_object = GenericFlatblock._default_manager.get(slug=slug)
             related_object = generic_object.content_object
+            if related_object is None:
+                # The related object must have been deleted. Let's start over.
+                generic_object.delete()
+                raise GenericFlatblock.DoesNotExist
         except GenericFlatblock.DoesNotExist:
             related_object = related_model._default_manager.create()
             generic_object = GenericFlatblock._default_manager.create(slug=slug, content_object=related_object)
@@ -81,11 +93,20 @@ class GenericFlatblockNode(Node):
 
         # Get the generic and related object
         generic_object, related_object = self.get_content_object(related_model, slug)
+        admin_url = self.generate_admin_link(related_object, context)
+
+        # if "into" is provided, store the related object into this variable
+        if self.store_in_object:
+            into_var = self.resolve(self.store_in_object, context)
+            context[into_var] = related_object
+            context["%s_generic_object" % into_var] = generic_object
+            context["%s_admin_url" % into_var] = admin_url
+            return ''
 
         # Add the model instances to the current context
         context['generic_object'] = generic_object
         context['object'] = related_object
-        context['admin_url'] = self.generate_admin_link(related_object, context)
+        context['admin_url'] = admin_url
 
         # Resolve the template(s)
         template_paths = []
@@ -110,9 +131,10 @@ class GenericFlatblockNode(Node):
 
 def do_genericflatblock(parser, token):
     """
-    {% genericflatblcok "slug" for "appname.modelname" %}
-    {% genericflatblcok "slug" for "appname.modelname" with "templatename.html" %}
-    {% genericflatblcok "slug" for "appname.modelname" with "templatename.html" as "variable" %}
+    {% gblock "slug" for "appname.modelname" %}
+    {% gblock "slug" for "appname.modelname" into "slug_object" %}
+    {% gblock "slug" for "appname.modelname" with "templatename.html" %}
+    {% gblock "slug" for "appname.modelname" with "templatename.html" as "variable" %}
     """
 
     def next_bit_for(bits, key, if_none=None):
@@ -127,6 +149,7 @@ def do_genericflatblock(parser, token):
         'modelname': next_bit_for(bits, 'for'),
         'template_path': next_bit_for(bits, 'with'),
         'variable_name': next_bit_for(bits, 'as'),
+        'store_in_object': next_bit_for(bits, 'into'),
     }
     return GenericFlatblockNode(**args)
 
